==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/light/classes/symbol/parameters/SymbolLightTypeParameterList.kt <==
/*
 * Copyright 2010-2024 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.light.classes.symbol.parameters

import ksp.com.intellij.psi.*
import ksp.com.intellij.psi.impl.light.LightElement
import ksp.com.intellij.psi.scope.PsiScopeProcessor
import ksp.org.jetbrains.kotlin.analysis.api.projectStructure.KaModule
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaDeclarationSymbol
import ksp.org.jetbrains.kotlin.analysis.api.symbols.pointers.KaSymbolPointer
import ksp.org.jetbrains.kotlin.analysis.api.symbols.typeParameters
import ksp.org.jetbrains.kotlin.asJava.classes.lazyPub
import ksp.org.jetbrains.kotlin.idea.KotlinLanguage
import ksp.org.jetbrains.kotlin.light.classes.symbol.*
import ksp.org.jetbrains.kotlin.light.classes.symbol.basicIsEquivalentTo
import ksp.org.jetbrains.kotlin.light.classes.symbol.compareSymbolPointers
import ksp.org.jetbrains.kotlin.light.classes.symbol.methods.SymbolLightMethodBase
import ksp.org.jetbrains.kotlin.light.classes.symbol.toArrayIfNotEmptyOrDefault
import ksp.org.jetbrains.kotlin.light.classes.symbol.withSymbol
import ksp.org.jetbrains.kotlin.psi.KtTypeParameterListOwner

internal class SymbolLightTypeParameterList(
    internal val owner: PsiTypeParameterListOwner,
    private val symbolWithTypeParameterPointer: KaSymbolPointer<KaDeclarationSymbol>,
    internal val ktModule: KaModule,
    private val ktDeclaration: KtTypeParameterListOwner?,
) : LightElement(owner.manager, KotlinLanguage.INSTANCE), PsiTypeParameterList {
    override fun accept(visitor: PsiElementVisitor) {
        if (visitor is JavaElementVisitor) {
            visitor.visitTypeParameterList(this)
        } else {
            visitor.visitElement(this)
        }
    }

    override fun processDeclarations(
        processor: PsiScopeProcessor,
        state: ResolveState,
        lastParent: PsiElement?,
        place: PsiElement
    ): Boolean = typeParameters.all { processor.execute(it, state) }

    private val _typeParameters: Collection<PsiTypeParameter> by lazyPub {
        symbolWithTypeParameterPointer.withSymbol(ktModule) {
            val parentInterface =
                (owner as? SymbolLightMethodBase)?.containingClass?.interfaceIfDefaultImpls

            val fromInterface = parentInterface?.typeParameters?.mapNotNull {
                (it as? SymbolLightTypeParameter)?.copyTo(this@SymbolLightTypeParameterList)
            }.orEmpty()

            fromInterface + it.typeParameters.mapIndexed { index, parameter ->
                SymbolLightTypeParameter(
                    ktAnalysisSession = this,
                    parent = this@SymbolLightTypeParameterList,
                    index = fromInterface.size + index,
                    typeParameterSymbol = parameter,
                )
            }
        }
    }

    override fun getTypeParameters(): Array<PsiTypeParameter> = _typeParameters.toArrayIfNotEmptyOrDefault(PsiTypeParameter.EMPTY_ARRAY)

    override fun getTypeParameterIndex(typeParameter: PsiTypeParameter?): Int = _typeParameters.indexOf(typeParameter)

    override fun toString(): String = "SymbolLightTypeParameterList"

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other !is SymbolLightTypeParameterList || other.ktModule != ktModule) return false
        if (ktDeclaration != null || other.ktDeclaration != null) {
            return other.ktDeclaration == ktDeclaration
        }

        return other.owner == owner && compareSymbolPointers(symbolWithTypeParameterPointer, other.symbolWithTypeParameterPointer)
    }

    override fun hashCode(): Int = ktDeclaration.hashCode() + 1

    override fun isEquivalentTo(another: PsiElement?): Boolean = basicIsEquivalentTo(this, another)

    override fun getParent(): PsiElement = owner
    override fun getContainingFile(): PsiFile = parent.containingFile
    override fun getText(): String? = ktDeclaration?.typeParameterList?.text
    override fun getTextOffset(): Int = ktDeclaration?.typeParameterList?.textOffset ?: -1
    override fun getStartOffsetInParent(): Int = ktDeclaration?.typeParameterList?.startOffsetInParent ?: -1
}

==> symbol-processing-aa-embeddable-2.3.0-sources.jar: org/jetbrains/kotlin/light/classes/symbol/parameters/SymbolLightTypeParameterList.kt <==
/*
 * Copyright 2010-2025 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.light.classes.symbol.parameters

import ksp.com.intellij.psi.*
import ksp.com.intellij.psi.impl.light.LightElement
import ksp.com.intellij.psi.scope.PsiScopeProcessor
import ksp.org.jetbrains.kotlin.analysis.api.projectStructure.KaModule
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaDeclarationSymbol
import ksp.org.jetbrains.kotlin.analysis.api.symbols.pointers.KaSymbolPointer
import ksp.org.jetbrains.kotlin.analysis.api.symbols.typeParameters
import ksp.org.jetbrains.kotlin.asJava.classes.lazyPub
import ksp.org.jetbrains.kotlin.idea.KotlinLanguage
import ksp.org.jetbrains.kotlin.light.classes.symbol.*
import ksp.org.jetbrains.kotlin.light.classes.symbol.methods.SymbolLightMethodBase
import ksp.org.jetbrains.kotlin.psi.KtTypeParameterListOwner

internal class SymbolLightTypeParameterList(
    internal val owner: PsiTypeParameterListOwner,
    private val symbolWithTypeParameterPointer: KaSymbolPointer<KaDeclarationSymbol>,
    internal val ktModule: KaModule,
    private val ktDeclaration: KtTypeParameterListOwner?,
) : LightElement(owner.manager, KotlinLanguage.INSTANCE), PsiTypeParameterList {
    override fun accept(visitor: PsiElementVisitor) {
        if (visitor is JavaElementVisitor) {
            visitor.visitTypeParameterList(this)
        } else {
            visitor.visitElement(this)
        }
    }

    override fun processDeclarations(
        processor: PsiScopeProcessor,
        state: ResolveState,
        lastParent: PsiElement?,
        place: PsiElement
    ): Boolean = typeParameters.all { processor.execute(it, state) }

    private val _typeParameters: Collection<PsiTypeParameter> by lazyPub {
        symbolWithTypeParameterPointer.withSymbol(ktModule) {
            val parentInterface =
                (owner as? SymbolLightMethodBase)?.containingClass?.interfaceIfDefaultImpls

            val fromInterface = parentInterface?.typeParameters?.mapNotNull {
                (it as? SymbolLightTypeParameter)?.copyTo(this@SymbolLightTypeParameterList)
            }.orEmpty()

            fromInterface + it.typeParameters.mapIndexed { index, parameter ->
                SymbolLightTypeParameter(
                    parent = this@SymbolLightTypeParameterList,
                    index = fromInterface.size + index,
                    typeParameterSymbol = parameter,
                )
            }
        }
    }

    override fun getTypeParameters(): Array<PsiTypeParameter> = _typeParameters.toArrayIfNotEmptyOrDefault(PsiTypeParameter.EMPTY_ARRAY)

    override fun getTypeParameterIndex(typeParameter: PsiTypeParameter?): Int = _typeParameters.indexOf(typeParameter)

    override fun toString(): String = this::class.simpleName.orEmpty()

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other !is SymbolLightTypeParameterList || other.ktModule != ktModule) return false
        if (ktDeclaration != null || other.ktDeclaration != null) {
            return other.ktDeclaration == ktDeclaration
        }

        return other.owner == owner && compareSymbolPointers(symbolWithTypeParameterPointer, other.symbolWithTypeParameterPointer)
    }

    override fun hashCode(): Int = ktDeclaration.hashCode() + 1

    override fun isEquivalentTo(another: PsiElement?): Boolean = basicIsEquivalentTo(this, another)

    override fun getParent(): PsiElement = owner
    override fun getContainingFile(): PsiFile = parent.containingFile
    override fun getText(): String? = ktDeclaration?.typeParameterList?.text
    override fun getTextOffset(): Int = ktDeclaration?.typeParameterList?.textOffset ?: -1
    override fun getStartOffsetInParent(): Int = ktDeclaration?.typeParameterList?.startOffsetInParent ?: -1
}
